//--------------------------------------------------------------------------------------------------
// Copyright (c) 2012, SiliconBlue Technologies, Inc.
//--------------------------------------------------------------------------------------------------
// Project       : I2S_Controller
//--------------------------------------------------------------------------------------------------
// File          : I2S_Controller.v
// Company       : SiliconBlue Technologies, Inc.
// Author        : MDN
// Created       : 14 February 2012
// Last modified : 14 February 2012
//--------------------------------------------------------------------------------------------------
// Description : This is the Top Module for connecting I2S Transmitter and Receiver. Selecting
//               appropriate value for the parameter I2S_Tx_Rx will decide whether to connect
//               Tx module or to connect Rx module.
// -------------------------------------------------------------------------------------------------


module I2S_Controller (
                       i_sys_clk,
                       i_sys_rst,
                       i_left_data,
                       i_right_data,
                       i_left_vld,
                       i_right_vld,
                       i_ws,
                       i_sd,
                       o_sck,
                       o_ws,
                       o_sd,
                       o_left_data,
                       o_right_data,
                       o_left_vld,
                       o_right_vld
                       );


    // I2S Tx Ports

    parameter I2S_Tx_Rx = 0;  
    parameter DATA_WIDTH= 16;
    
    input                      i_sys_clk;//System clock
    input                      i_sys_rst;//System reset, asynchronous active high

    input [DATA_WIDTH - 1 : 0] i_left_data;//Input data bus - carries data to be sent via i2s bus - Left Channel Data
    input [DATA_WIDTH - 1 : 0] i_right_data;//Input data bus - carries data to be sent via i2s bus
    input                      i_left_vld;//A high indicates existence of left channel data on the input bus 
    input                      i_right_vld;//A high indicates existence of right channel data on the input bus
    
    output                     o_sck;//Serial clock line
    output                     o_ws;//Write Select,a high indicates right channel and low indicates left channel
    output                     o_sd;//Serial data

    // I2S Rx Ports
        
    input i_ws;//I2S write select, high indicates right channel and low indicates left
    input i_sd;//I2S serial data
    
    output [DATA_WIDTH - 1 : 0] o_left_data;//Output left channel data to Host/Processor
    output [DATA_WIDTH - 1 : 0] o_right_data;//Output right channel data to Host/Processor
    output                   o_left_vld;
    output                   o_right_vld;

    

    
    generate
        if (I2S_Tx_Rx == 1)
            i2s_tx #(
                     .DATA_WIDTH(DATA_WIDTH)
                     )
            i2s_tx_inst
                (
                 .i_sys_clk(i_sys_clk),
                 .i_sys_rst(i_sys_rst),
                 .i_left_data(i_left_data),
                 .i_right_data(i_right_data),
                 .i_left_vld(i_left_vld),
                 .i_right_vld(i_right_vld),     
                 .o_sck(o_sck),
                 .o_ws(o_ws),
                 .o_sd(o_sd)
                 );
        else
            i2s_rx # (
                      .DATA_WIDTH(DATA_WIDTH)
                     )
            i2s_rx_inst
                (
                 .i_sys_rst(i_sys_rst),
                 .i_sck(i_sys_clk),
                 .i_ws(i_ws),
                 .i_sd(i_sd),
                 .o_left_data(o_left_data),
                 .o_right_data(o_right_data),
                 .o_left_vld(o_left_vld),
                 .o_right_vld(o_right_vld)
                 );
        endgenerate
    
endmodule
                     
                         